;;;
;;; package.lisp
;;;
;;; package definition for lh-linuxapi.
;;;

(cl:defpackage :lh-linuxapi
  (:use :cl :sb-alien)
  (:export

   ;; Types.
   "PID-T"

   ;; Stuff to do with wait(2) and friends.
   "P-ALL" "P-PID" "P-PGID"
   "WNOHANG" "WUNTRACED"
   "WSTOPPED" "WEXITED" "WCONTINUED" "WNOWAIT"
   "__WCLONE" "__WALL" "__WNOTHREAD"
   "WEXITSTATUS" "WTERMSIG" "WSTOPSIG" "WCOREDUMP"
   "WIFEXITED" "WIFSIGNALED" "WIFSTOPPED" "WIFCONTINUED"
   "PARSE-WAIT-STATUS"
   "WAIT" "WAITPID"

   ;; Stuff to do with ptrace(2).
   "PTRACE-TRACEME"
   "PTRACE-PEEKTEXT" "PTRACE-PEEKDATA" "PTRACE-PEEKUSER"
   "PTRACE-POKETEXT" "PTRACE-POKEDATA" "PTRACE-POKEUSER"
   "PTRACE-CONT" "PTRACE-KILL" "PTRACE-SINGLESTEP"
   "PTRACE-ATTACH" "PTRACE-DETACH"
   "PTRACE-SYSCALL"
   "PTRACE-SETOPTIONS" "PTRACE-GETEVENTMSG"
   "PTRACE-GETSIGINFO" "PTRACE-SETSIGINFO"
   "PTRACE-O-TRACESYSGOOD" "PTRACE-O-TRACEFORK" "PTRACE-O-TRACEVFORK"
   "PTRACE-O-TRACECLONE" "PTRACE-O-TRACEEXEC" "PTRACE-O-TRACEVFORKDONE"
   "PTRACE-O-TRACEEXIT" "PTRACE-O-MASK"
   "PTRACE-EVENT-FORK" "PTRACE-EVENT-VFORK" "PTRACE-EVENT-CLONE"
   "PTRACE-EVENT-EXEC" "PTRACE-EVENT-VFORK-DONE" "PTRACE-EVENT-EXIT"
   "PTRACE-GETREGS" "PTRACE-SETREGS" "PTRACE-GETFPREGS" "PTRACE-SETFPREGS"
   "PTRACE-GETFPXREGS" "PTRACE-SETFPXREGS"
   "PTRACE-OLDSETOPTIONS"
   "PTRACE-SYSEMU" "PTRACE-SYSEMU-SINGLESTEP"
   "PTRACE-SINGLEBLOCK"
   "PTRACE"
   ))

;;; EOF
