;;;
;;; package.lisp
;;;
;;; Package definition for lh-debug-core.
;;;

(cl:defpackage :lh-debug-core
  (:use :cl :sb-alien :lh-linuxapi)
  (:export

   ;; What?
   "EVENT-FUNCTION"
   "UPDATE-PROCESS-STATUS"
   "DETACH-FROM-PROCESS"
   "RESUME-PROCESS"
   "ATTACH-TO-PROCESS"
   "REGISTER-VALUE"
   ))

;;; EOF
